#
#  activity_spots.py
#  
#
#  Created by Dumusque Xavier on 2/17/09.
#  Copyright (c) 2009 __MyCompanyName__. All rights reserved.
#

# Generating spot of maximal size according to the observed distribution on the Sun. The lifetime of spot are then different. The apparition of spots is ruled by a Poisson law

import matplotlib
import matplotlib.animation as manimation
from pylab import *
import os,sys,string
import glob
import random
import shutil
import pyfits
import math

sys.path.append('functions/')

import pyrdb
import periodogram


#######################################################################################
###                                PARAMETERS                                       ###
#######################################################################################

#Exponent of the poisson law for the appearence of spots
lambda_48 = 6. # frequence [1/day]
initial_time_to_generate_spot = 30. # time in days where we generate the first spots, before evolution with a Poisson law


#######################################################################################
###                                FUNCTIONS                                        ###
#######################################################################################

## GLS, Zechmeister 2009
def compute_periodogram(x,y,sig_y=0,number_of_night=1,ofac=1.):
    
    ## x and y must be float vectors
    x = array(x)*1.
    y = array(y)*1.
    
    if type(sig_y) == int:
        weight=ones(len(x))/len(x)
        W = sum(weight)
    else:
        W = sum(1./array(sig_y)**2)
        weight=1./W * 1./array(sig_y)**2
    
    median_step=median(x[1:]-x[:-1])
    hifac = median_step

    freq,power,phi,a_cos,b_sin,c_cte,fmax,fap,z0,z01,variance = periodogram.periodogram(x,y,weight,ofac,hifac,number_of_night)
    
    return freq,power,phi


def fact(x):
    if x==0:
        return 1
    else:
        return x * fact(x-1)

#Spherical law of cosine http://en.wikipedia.org/wiki/Spherical_law_of_cosines
def law_cosine(lon,lat,angle):
    #cos(c) = cos(lon*pi/180.)*cos(lat*pi/180.)+sin(lon*pi/180.)*sin(lat*pi/180.)*cos(angle*pi/180.)
    #c = math.acos(cos(lon*pi/180.)*cos(lat*pi/180.)+sin(lon*pi/180.)*sin(lat*pi/180.)*cos(angle*pi/180.))
    #c * 180./pi in degrees
    return math.acos(cos(lon*pi/180.)*cos(lat*pi/180.)+sin(lon*pi/180.)*sin(lat*pi/180.)*cos(angle*pi/180.))*180./pi

#Relation to convert spot size in spot number (see Hattaway 2000)
def spot_area_to_number(spot_area):
    return 0.04*array(spot_area)


#Give the position of active longitudes. We consider here 4 active longitues
def active_longitude_position(bb,t0_spot,rotation_period):
    
    if bb<-0.25:
        position_active_long = (t0_spot*360/rotation_period+0)%360
    elif bb>=-0.25 and bb<0.:
        position_active_long = (t0_spot*360/rotation_period+180)%360
    elif bb>=0. and bb<0.25:
        position_active_long = (t0_spot*360/rotation_period+90)%360
    elif bb>=0.25:
        position_active_long = (t0_spot*360/rotation_period+270)%360
    
    return position_active_long


#To get the lifetime of a spot group
def spot_lifetime_func(sum_spot_number,t0_spot,t):
    
    #randomly choose the lifetime of a spot
    b=random.random()
    if b<0.5:
        lifetime=(random.random()*47+1)/24. # in days 1h<T<2d, 50 % of the spots
    elif b>0.9:
        lifetime=random.random()*49+11 # in days 11d<T<2month, 10% of the spots
    else:
        lifetime=random.random()*9+2 # in days 2d<T<11d, 40% of the spots
    
    #To reject spot that last for less than 1 day
    if lifetime < 1:
        sum_spot_number.append(sum_spot_number[-1]+zeros(len(t)))
        
        return sum_spot_number,0.,0.,0.,lifetime
    
    #If the spot group has a lifetime longer than 1 day
    else:
        
        #calculate maximum filling factor given the spot group lifetime
        max_filling_factor=lifetime*1e-5*1e6 #in micro hemisphere, see Dumusque 2011b, max_ff = 1e-5*T in hemisphere, to have it in mirco hemisphere, max_ff = 1e-5*T*1e6
        
        # Sopt groups increase their size during 1/3 of their lifetime and they decrease until they disappear during 2/3 of the lifetime
        increase_period=lifetime/3.
        decrease_period=lifetime-increase_period
        
        pente_increase=max_filling_factor/increase_period
        origin_increase=0-pente_increase*(t0_spot) # b=y-ax
        pente_decrease=-max_filling_factor/decrease_period
        origin_decrease=max_filling_factor-pente_decrease*(increase_period+t0_spot) # b=y-ax
        
        #Calculate the spot group area as a function of time
        spot_area, spot_time = [], []
        for i in arange(len(t)):
            
            if t[i]-t0_spot>0 and t[i]-t0_spot<=lifetime:
                
                if t[i]-t0_spot>=0 and t[i]-t0_spot<increase_period:
                    spot_area = append(spot_area,pente_increase*t[i]+origin_increase)
                else:
                    spot_area = append(spot_area,pente_decrease*t[i]+origin_decrease)
                
                spot_time = append(spot_time,t[i])
        
        return sum_spot_number,max_filling_factor,spot_area,spot_time,lifetime


# Give the RV, the BIS and the FWHM induced by a spot of a given size (based on SOAP simualtion)        
def active_region_effect(phase_vect,spot_area,spot_time,max_filling_factor,latitude,longitude,spot,plage,ratio_plage_spot):
    
    active_region_prop = spot.lat_15_00_phase_0_000
    active_region_prop_vect_spot = dict([(ii,[]) for ii in active_region_prop.keys()])
    active_region_prop_vect_plage = dict([(ii,[]) for ii in active_region_prop.keys()])
    active_region_prop_vect_small_plage = dict([(ii,[]) for ii in active_region_prop.keys()])
    
    x_spot=sin((90-latitude)*pi/180.)*cos(longitude*pi/180.)
    y_spot=sin((90-latitude)*pi/180.)*sin(longitude*pi/180.)
    z_spot=cos((90-latitude)*pi/180.)

    spot_area_projected = zeros(len(spot_area))
    select_spot_visible = (longitude%360 < 90.) | (longitude%360 > 270.)
    #cosine rule for spherical trigonometry, see Wiki. cos c= cos a*cos b + sin a*sin b*cos C
    #angle of the spot relative to the line of site
    cos_spot_angle_to_center = cos(latitude[select_spot_visible]*pi/180.)*cos(longitude[select_spot_visible]*pi/180.) + sin(latitude[select_spot_visible]*pi/180.)*sin(longitude[select_spot_visible]*pi/180.)*cos(90*pi/180.)
    spot_area_projected[select_spot_visible] = spot_area[select_spot_visible] * cos_spot_angle_to_center**2 # The spot area goes like cos(i)**2, i being the angle between the spot center and the center of the stellar disc.
    
    # For a radius of 1, the area of the hemisphere is 2pi. Therefore we have to divide the area of the spot (in hemisphere) by 2pi if we want the good scaling ratio.
    # NOT TOTALLY RIGHT BECAUSE WE PLOT IN 2D and not 3D, THE SIZE OF THE SPOTS/PLAGES ON THE LIMBS ARE NOT CORRECT
#    spot_radius  = sqrt(spot_area/1.e6/(2*pi))
#    plage_radius = sqrt((ratio_plage_spot+1)*spot_area/1.e6/(2*pi))
    spot_radius  = sqrt(2*spot_area/1.e6)
    plage_radius = sqrt(2*(ratio_plage_spot+1)*spot_area/1.e6)


    spot_number_visible = zeros(len(spot_area))
    spot_number = spot_area_to_number(spot_area)
    spot_number_visible[select_spot_visible] = spot_area_to_number(spot_area[select_spot_visible])

    active_region_prop_vect_spot.update({'x_spot':x_spot,'y_spot':y_spot,'z_spot':z_spot,'spot_area':spot_area,'spot_radius':spot_radius,'spot_area_projected':spot_area_projected,'spot_time':spot_time,'spot_number':spot_number,'spot_number_visible':spot_number_visible,'max_filling_factor':max_filling_factor,'latitude':latitude,'longitude':longitude})
    active_region_prop_vect_plage.update({'x_spot':x_spot,'y_spot':y_spot,'z_spot':z_spot,'spot_area':(ratio_plage_spot+1)*spot_area,'plage_radius':plage_radius,'spot_area_projected':(ratio_plage_spot+1)*spot_area_projected,'spot_time':spot_time,'spot_number':spot_number,'spot_number_visible':spot_number_visible,'max_filling_factor':max_filling_factor,'latitude':latitude,'longitude':longitude})
    active_region_prop_vect_small_plage.update({'x_spot':x_spot,'y_spot':y_spot,'z_spot':z_spot,'spot_area':spot_area,'plage_radius':spot_radius,'spot_area_projected':spot_area_projected,'spot_time':spot_time,'spot_number':spot_number,'spot_number_visible':spot_number_visible,'max_filling_factor':max_filling_factor,'latitude':latitude,'longitude':longitude})

    
    lat_simulated = array([15.0]) #latitude that have been simulated with SOAP 2.0
    lat_selected = lat_simulated[where(abs(abs(latitude[0])-lat_simulated) == min(abs(abs(latitude[0])-lat_simulated)))[0][0]] # Select the closest latitude that have been simulated with SOAP_new_with_Rhk.rdb
    
    phase=longitude/360.%1
    
    for ii in arange(len(phase)):
    
        if  phase[ii]>0.9975:
            phase_selected=0
        else:
            phase_selected = phase_vect[where(abs(phase[ii]-phase_vect)<0.0025)[0][0]]
        
        str_phase = string.replace('%.3f'%phase_selected,'.','_')
        str_lat = string.replace('%.2f'%lat_selected,'.','_')
        active_region_prop_spot = eval('spot.lat_%s_phase_%s'%(str_lat,str_phase))
        active_region_prop_plage = eval('plage.lat_%s_phase_%s'%(str_lat,str_phase))
        active_region_prop_small_plage = eval('plage.lat_%s_phase_%s'%(str_lat,str_phase))
        
        #In Dumusque et al. 2014 I show that we have a linearity in size for VR, BIS SPAN and FWHM
        active_region_ratio_spot = spot_area[ii]/spot.size_area
        active_region_ratio_plage = (ratio_plage_spot+1)*spot_area[ii]/plage.size_area
        active_region_ratio_small_plage = spot_area[ii]/plage.size_area
        
        #I show in Dumusque et al. 2014 that the amplitude in flux, VR, BIS SPAN et FWHM is proportional to cos(latitude)^2
        #Not totally sure for a plage because of limb-brightening, but not a big error in any case
        active_region_lat_ratio = cos(latitude[0]*pi/180.)**2/cos(lat_selected*pi/180.)**2
        
        for jj in ['vrad', 'bis_span', 'fwhm']:
            active_region_prop_vect_spot[jj]        = append(active_region_prop_vect_spot[jj],            (active_region_ratio_spot*active_region_lat_ratio)*active_region_prop_spot[jj]*1000) #in m/s
            active_region_prop_vect_plage[jj]       = append(active_region_prop_vect_plage[jj],           (active_region_ratio_plage*active_region_lat_ratio)*active_region_prop_plage[jj]*1000) #in m/s
            active_region_prop_vect_small_plage[jj] = append(active_region_prop_vect_small_plage[jj],     (active_region_ratio_small_plage*active_region_lat_ratio)*active_region_prop_small_plage[jj]*1000) #in m/s
        active_region_prop_vect_spot['flux']        = append(active_region_prop_vect_spot['flux'],        (active_region_ratio_spot*active_region_lat_ratio)*active_region_prop_spot['flux']*1e6) #in microhemisphere
        active_region_prop_vect_plage['flux']       = append(active_region_prop_vect_plage['flux'],       (active_region_ratio_plage*active_region_lat_ratio)*active_region_prop_plage['flux']*1e6) #in microhemisphere
        active_region_prop_vect_small_plage['flux'] = append(active_region_prop_vect_small_plage['flux'], (active_region_ratio_small_plage*active_region_lat_ratio)*active_region_prop_small_plage['flux']*1e6) #in microhemisphere
            
    return active_region_prop_vect_spot,active_region_prop_vect_plage,active_region_prop_vect_small_plage
        
        
#######################################################################################
###                                  MAIN                                           ###
#######################################################################################


####################################################
##     INITIALIZATION OF SPOT POSITION AND NUMBER
####################################################

def init_simulation(initial_spot_number,rotation_period,phase_vect,t,sampling_t,latitude_active_region,dispersion_longitude_AL,dispersion_latitude_AL,ratio_plage_spot,SPOT,PLAGE):
    
    initial_longitude_spot,initial_latitude_spot = [], []
    spot_effect_vect,plage_effect_vect = [],[]
    spot_number_vect,spot_number_vect_all = [],[]
    sum_spot_number=[zeros(len(t))]
    
    spot = SPOT()
    plage = PLAGE()
    
    while max(sum_spot_number[-1])<initial_spot_number:

        sys.stdout.flush()
        sys.stdout.write("\r Number of spot generated so far = %d" % max(sum_spot_number[-1]))

        t0_spot=round(initial_time_to_generate_spot*random.random()/sampling_t)*sampling_t # time when the spot appears

        # Spot properties (size, location) as a function of the distribution defined by HOWARD 2000 (Encyclopeadia of the Universe or Astronomy)
        sum_spot_number,max_filling_factor,spot_area,spot_time,spot_lifetime = spot_lifetime_func(sum_spot_number,t0_spot,t)

        #If the spot lifetime is more than a day, consider the spot
        if spot_lifetime >= 1:
            
            bb = random.random()-0.5
            initial_latitude_spot.append(sign(bb)*random.gauss(latitude_active_region,dispersion_latitude_AL)) # Spot in the southern or northern hemisphere given the sign of bb
            
            #Give the active longitude where the spot will be
            position_active_long = active_longitude_position(bb,t0_spot,rotation_period)
            initial_longitude_spot.append(position_active_long+random.gauss(0,dispersion_longitude_AL))
            
            #Check if the new spot is overlapping previous generated spots at the same time
            if len(spot_effect_vect) > 0:
                index_active_spots,new_spot_forbiden_long,new_spot_forbiden_lat = array([],'int'),[],[]
                for k in arange(len(spot_effect_vect)):
                    if t0_spot < max(spot_effect_vect[k]['spot_time']) or t0_spot+spot_lifetime > min(spot_effect_vect[k]['spot_time']):
                        index_active_spots = append(index_active_spots,k)
                        initial_time_diff = t0_spot - spot_effect_vect[k]['spot_time'][0] #time difference between the times where the spots appears
                        initial_longitude_diff = initial_time_diff*360/rotation_period #longitude difference between the times where the spots appears
                        spot_longitude_t0 = initial_longitude_diff+spot_effect_vect[k]['longitude'][0]
                        spot_radius_angle = max(spot_effect_vect[k]['spot_radius']) *  360/(2*pi) #angle of the spot radius (spots already present), times 360 / (2*pi*Rstar) to get from radius to angle, Rstar being 1 by definition
                        new_spot_radius_angle = sqrt(2*max_filling_factor/1.e6) *  360/(2*pi) #angle of the spot radius (new spot to put on the star), times 360 / (2*pi*Rstar) to get from radius to angle, Rstar being 1 by definition
                        new_spot_forbiden_long = append(new_spot_forbiden_long,arange(round(spot_longitude_t0-(spot_radius_angle+new_spot_radius_angle),1),spot_longitude_t0+(spot_radius_angle+new_spot_radius_angle),0.1)%360)
                        new_spot_forbiden_lat = append(new_spot_forbiden_lat,arange(round(spot_effect_vect[k]['latitude'][0]-(spot_radius_angle+new_spot_radius_angle),1),spot_effect_vect[k]['latitude'][0]+(spot_radius_angle+new_spot_radius_angle),0.1)) #[x_min,x_max],[y_min,y_max]

                if round(initial_longitude_spot[-1],1) in new_spot_forbiden_long or round(initial_latitude_spot[-1],1) in new_spot_forbiden_lat:
                    bb = random.random()-0.5
                    position_active_long = active_longitude_position(bb,t0_spot,rotation_period)
                    initial_longitude_spot[-1] = position_active_long+random.gauss(0,dispersion_longitude_AL)
                    initial_latitude_spot[-1]  = sign(bb)*random.gauss(latitude_active_region,dispersion_latitude_AL)

            latitude=ones(len(spot_time)) * initial_latitude_spot[-1]
            longitude=(array(spot_time)-t0_spot)*360/rotation_period+initial_longitude_spot[-1]
                        
            spot_effect,plage_effect,small_plage_effect   = active_region_effect(phase_vect,spot_area,spot_time,max_filling_factor,latitude,longitude,spot,plage,ratio_plage_spot)
            for keys in ['flux','vrad', 'bis_span', 'fwhm','spot_area','spot_area_projected','spot_number','max_filling_factor','latitude']:
                plage_effect[keys] = plage_effect[keys]-small_plage_effect[keys] #to suppress the middle of the plage that is affected by the spot
            spot_effect_vect.append(spot_effect)
            plage_effect_vect.append(plage_effect)
                                    
            spot_number_vect_all.append(zeros(len(t)))
            index0 = int(spot_effect_vect[-1]['spot_time'][0]/sampling_t)
            len_vect = len(spot_effect_vect[-1]['spot_time'])
            spot_number_vect_all[-1][index0:index0+len_vect] = spot_effect_vect[-1]['spot_number']
            sum_spot_number.append(array(sum_spot_number[-1])+spot_number_vect_all[-1]) # We sum all the spot that are present at the same moment

    print
    print
    print 'Initialization finished'
    print

    return initial_longitude_spot,initial_latitude_spot,spot_effect_vect,plage_effect_vect,spot_number_vect,spot_number_vect_all,sum_spot_number

#########################################
##     TIME EVOLUTION OF SPOTS
#########################################

def spot_evolution(rotation_period,phase_vect,t,sampling_t,max_t,latitude_active_region,dispersion_longitude_AL,dispersion_latitude_AL,ratio_plage_spot,initial_longitude_spot,initial_latitude_spot,spot_effect_vect,plage_effect_vect,spot_number_vect,spot_number_vect_all,sum_spot_number,SPOT,PLAGE):
    
    spot = SPOT()
    plage = PLAGE()

    for l in arange(initial_time_to_generate_spot/sampling_t,len(t)-2,1):
        
        sys.stdout.flush()
        sys.stdout.write("\r time =  %i days over %i" % (l*sampling_t+1,max_t))

        proba=[]
        # Poisson probability of spot appearence
        for m in arange(20):
            proba.append(exp(-lambda_48*sampling_t)*(lambda_48*sampling_t)**m/fact(m))
        a=random.random()
        nb_new_spots = where(cumsum(proba)>a)[0][0]

        for o in arange(nb_new_spots):

            t0_spot=t[int(l)]

            # Spot properties (size, location) as a function of the distribution defined by HOWARD 2000 (Encyclopeadia of the Universe or Astronomy)
            sum_spot_number,max_filling_factor,spot_area,spot_time,spot_lifetime = spot_lifetime_func(sum_spot_number,t0_spot,t)
            
            #If the spot lifetime is more than a day, consider the spot
            if spot_lifetime >= 1:

                bb = random.random()-0.5
                initial_latitude_spot.append(sign(bb)*random.gauss(latitude_active_region,dispersion_latitude_AL)) # Spot in the southern or northern hemisphere given the sign of bb

                #Give the active longitude where will be the spot
                position_active_long = active_longitude_position(bb,t0_spot,rotation_period)
                initial_longitude_spot.append(position_active_long+random.gauss(0,dispersion_longitude_AL))
            
                #Check if the new spot is overlapping previous generated spots at the same time
                if len(spot_effect_vect) > 0:
                    index_active_spots,new_spot_forbiden_long,new_spot_forbiden_lat = array([],'int'),[],[]
                    for k in arange(len(spot_effect_vect)):
                        if t0_spot < max(spot_effect_vect[k]['spot_time']) or t0_spot+spot_lifetime > min(spot_effect_vect[k]['spot_time']):
                            index_active_spots = append(index_active_spots,k)
                            initial_time_diff = t0_spot - spot_effect_vect[k]['spot_time'][0] #time difference between the times where the spots appears
                            initial_longitude_diff = initial_time_diff*360/rotation_period #longitude difference between the times where the spots appears
                            spot_longitude_t0 = initial_longitude_diff+spot_effect_vect[k]['longitude'][0]
                            spot_radius_angle = max(spot_effect_vect[k]['spot_radius']) *  360/(2*pi) #angle of the spot radius (spots already present), times 360 / (2*pi*Rstar) to get from radius to angle, Rstar being 1 by definition
                            new_spot_radius_angle = sqrt(2*max_filling_factor/1.e6) *  360/(2*pi) #angle of the spot radius (new spot to put on the star), times 360 / (2*pi*Rstar) to get from radius to angle, Rstar being 1 by definition
                            new_spot_forbiden_long = append(new_spot_forbiden_long,arange(round(spot_longitude_t0-(spot_radius_angle+new_spot_radius_angle),1),spot_longitude_t0+(spot_radius_angle+new_spot_radius_angle),0.1)%360)
                            new_spot_forbiden_lat = append(new_spot_forbiden_lat,arange(round(spot_effect_vect[k]['latitude'][0]-(spot_radius_angle+new_spot_radius_angle),1),spot_effect_vect[k]['latitude'][0]+(spot_radius_angle+new_spot_radius_angle),0.1)) #[x_min,x_max],[y_min,y_max]
                
                    if round(initial_longitude_spot[-1],1) in new_spot_forbiden_long or round(initial_latitude_spot[-1],1) in new_spot_forbiden_lat:
                        bb = random.random()-0.5
                        position_active_long = active_longitude_position(bb,t0_spot,rotation_period)
                        initial_longitude_spot[-1] = position_active_long+random.gauss(0,dispersion_longitude_AL)
                        initial_latitude_spot[-1]  = sign(bb)*random.gauss(latitude_active_region,dispersion_latitude_AL)

                latitude=ones(len(spot_time)) * initial_latitude_spot[-1]
                longitude=(array(spot_time)-t0_spot)*360/rotation_period+initial_longitude_spot[-1]

                spot_effect,plage_effect,small_plage_effect   = active_region_effect(phase_vect,spot_area,spot_time,max_filling_factor,latitude,longitude,spot,plage,ratio_plage_spot)
                for keys in ['flux','vrad', 'bis_span', 'fwhm','spot_area','spot_area_projected','spot_number','max_filling_factor','latitude']:
                    plage_effect[keys] = plage_effect[keys]-small_plage_effect[keys] #to suppress the middle of the plage that is affected by the spot
                spot_effect_vect.append(spot_effect)
                plage_effect_vect.append(plage_effect)

    print
    print
    print 'Finished'
    print

    return spot_effect_vect,plage_effect_vect


#########################################
##    CALCULATE SUM OBSERVABLES
#########################################

def format_outputs(spot_effect_vect,plage_effect_vect,t,sampling_t,number_of_active_longitude,SPOT,PLAGE):
    
    spot = SPOT()
    plage = PLAGE()

    nb_active_region = len(spot_effect_vect)
    number_of_active_longitude_vect = ones(len(t))*number_of_active_longitude

    active_region_effect_keys = ['flux','vrad', 'bis_span', 'fwhm','spot_area','spot_area_projected','spot_number','spot_number_visible','latitude','longitude','x_spot','y_spot','z_spot']

    nul_vect = array([0.])
    spot_effect_vect_all = dict([(ii,resize(nul_vect,(nb_active_region,len(t)))) for ii in active_region_effect_keys])
    plage_effect_vect_all = dict([(ii,resize(nul_vect,(nb_active_region,len(t)))) for ii in active_region_effect_keys])

    for i in arange(nb_active_region):
        
        sys.stdout.flush()
        sys.stdout.write("\r formatting spot group %i out of %i spot groups" % (i+1,nb_active_region))

        index0 = int(spot_effect_vect[i]['spot_time'][0]/sampling_t)
        len_vect = len(spot_effect_vect[i]['spot_time'])
        
        for keys in active_region_effect_keys:
            spot_effect_vect_all[keys][i][index0:index0+len_vect] = spot_effect_vect[i][keys]
            plage_effect_vect_all[keys][i][index0:index0+len_vect] = plage_effect_vect[i][keys]

    sum_active_region_keys = ['flux','vrad', 'bis_span', 'fwhm','spot_area','spot_area_projected','spot_number','spot_number_visible']

    print

    sum_spot_effect = dict([(ii,[]) for ii in sum_active_region_keys])
    sum_plage_effect = dict([(ii,[]) for ii in sum_active_region_keys])
        
    for i,keys in enumerate(sum_active_region_keys):
        
        sys.stdout.flush()
        sys.stdout.write("\r formatting %i out of %i time series" % (i+1,len(sum_active_region_keys)))
        
        sum_spot_effect[keys] = array([sum(spot_effect_vect_all[keys][:,ii]) for ii in arange(len(t))])
        sum_plage_effect[keys] = array([sum(plage_effect_vect_all[keys][:,ii]) for ii in arange(len(t))])
        if keys in ['flux', 'bis_span', 'fwhm']:
            sum_spot_effect[keys] += eval('spot.'+keys+'_star')
            sum_plage_effect[keys] += eval('plage.'+keys+'_star')
        
    print
    print 'Finished output formating'
    print

    return number_of_active_longitude_vect,sum_spot_effect,sum_plage_effect,spot_effect_vect_all,plage_effect_vect_all

#########################################
##    FIGURES
#########################################

def plot_figures_and_save_file(number_of_active_longitude_vect,sum_spot_effect,sum_plage_effect,spot_effect_vect,plage_effect_vect,spot_effect_vect_all,plage_effect_vect_all,t,rotation_period,inclinaison):

    figure(1)
    subplot(311)
    title('Spot number visible and filling factor')
    plot(t,sum_spot_effect['spot_number_visible'],rasterized=True)
    axhline(mean(sum_spot_effect['spot_number_visible']),rasterized=True)
    ylabel('# spots')
    ax = gca()
    setp(ax.get_xticklabels(), visible=False)
    subplot(312,sharex=ax)
    plot(t,sum_spot_effect['spot_area_projected']*1e-6*100,rasterized=True)
    axhline(mean(sum_spot_effect['spot_area_projected']*1e-6*100),rasterized=True)
    ylabel('Spot ff [%]')
    ax=gca()
    setp(ax.get_xticklabels(), visible=False)
    subplot(313,sharex=ax)
    plot(t,sum_plage_effect['spot_area_projected']*1e-6*100,rasterized=True)
    axhline(mean(sum_plage_effect['spot_area_projected']*1e-6*100),rasterized=True)
    xlabel('time [d]')
    ylabel('Plage ff [%]')
    subplots_adjust(top=0.92,left=0.15,bottom=0.15,right=0.92,wspace=0.10,hspace=0.10)
    savefig('figures/spot_number_and_area_coverage_log_Rhk_48_period_'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'.pdf')

    figure(2)
    subplot(211)
    title('Spot photometric and radial velocity effect')
    observable = 'flux'
    plot(t,sum_spot_effect[observable],color='b',label='spot',rasterized=True)
    plot(t,sum_plage_effect[observable],color='g',label='plage',rasterized=True)
    plot(t,sum_spot_effect[observable]+sum_plage_effect[observable],color='r',label='tot',rasterized=True)
    ylabel('Flux [ppm]')
    legend()
    ax = gca()
    setp(ax.get_xticklabels(), visible=False)
    subplot(212,sharex=ax)
    observable = 'vrad'
    plot(t,sum_spot_effect[observable],color='b',label='spot',rasterized=True)
    plot(t,sum_plage_effect[observable],color='g',label='plage',rasterized=True)
    plot(t,sum_spot_effect[observable]+sum_plage_effect[observable],color='r',label='tot',rasterized=True)
    ylabel('RV [m/s]')
    xlabel('time [d]')
    ax = gca()
    subplots_adjust(top=0.92,left=0.15,bottom=0.15,right=0.92,wspace=0.10,hspace=0.10)
    savefig('figures/Flux_and_RV_effect_log_Rhk_48_period_'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'.pdf')

    figure(3,[16,14])
    subplot(331)
    title('SPOT')
    plot(sum_spot_effect['vrad'],sum_spot_effect['bis_span'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel('BIS SPAN [m/s]')
    subplot(334)
    plot(sum_spot_effect['vrad'],sum_spot_effect['fwhm'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel('FWHM [m/s]')
    subplot(337)
    plot(sum_spot_effect['vrad'],sum_spot_effect['flux'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel("Flux [ppm]")
    subplot(335)
    plot(sum_spot_effect['bis_span'],sum_spot_effect['fwhm'],'.',rasterized=True)
    xlabel('BIS SPAN [m/s]')
    ylabel('FWHM [m/s]')
    subplot(338)
    plot(sum_spot_effect['bis_span'],sum_spot_effect['flux'],'.',rasterized=True)
    xlabel('BIS SPAN [m/s]')
    ylabel("Flux [ppm]")
    subplot(339)
    plot(sum_spot_effect['fwhm'],sum_spot_effect['flux'],'.',rasterized=True)
    xlabel('FWHM [m/s]')
    ylabel("Flux [ppm]")
    subplots_adjust(top=0.98,left=0.1,bottom=0.1,right=0.98,wspace=0.32,hspace=0.32)
    savefig('figures/SPOT_Correlations_log_Rhk_48_period_'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'.pdf')

    figure(301,[16,14])
    subplot(331)
    title('PLAGE')
    plot(sum_plage_effect['vrad'],sum_plage_effect['bis_span'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel('BIS SPAN [m/s]')
    subplot(334)
    plot(sum_plage_effect['vrad'],sum_plage_effect['fwhm'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel('FWHM [m/s]')
    subplot(337)
    plot(sum_plage_effect['vrad'],sum_plage_effect['flux'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel("Flux [ppm]")
    subplot(335)
    plot(sum_plage_effect['bis_span'],sum_plage_effect['fwhm'],'.',rasterized=True)
    xlabel('BIS SPAN [m/s]')
    ylabel('FWHM [m/s]')
    subplot(338)
    plot(sum_plage_effect['bis_span'],sum_plage_effect['flux'],'.',rasterized=True)
    xlabel('BIS SPAN [m/s]')
    ylabel("Flux [ppm]")
    subplot(339)
    plot(sum_plage_effect['fwhm'],sum_plage_effect['flux'],'.',rasterized=True)
    xlabel('FWHM [m/s]')
    ylabel("Flux [ppm]")
    subplots_adjust(top=0.98,left=0.1,bottom=0.1,right=0.98,wspace=0.32,hspace=0.32)
    savefig('figures/PLAGE_Correlations_log_Rhk_48_period_'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'.pdf')

    figure(302,[16,14])
    subplot(331)
    title('SPOT + PLAGE')
    plot(sum_spot_effect['vrad']+sum_plage_effect['vrad'],sum_spot_effect['bis_span']+sum_plage_effect['bis_span'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel('BIS SPAN [m/s]')
    subplot(334)
    plot(sum_spot_effect['vrad']+sum_plage_effect['vrad'],sum_spot_effect['fwhm']+sum_plage_effect['fwhm'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel('FWHM [m/s]')
    subplot(337)
    plot(sum_spot_effect['vrad']+sum_plage_effect['vrad'],sum_spot_effect['flux']+sum_plage_effect['flux'],'.',rasterized=True)
    xlabel('RV [m/s]')
    ylabel("Flux [ppm]")
    subplot(335)
    plot(sum_spot_effect['bis_span']+sum_plage_effect['bis_span'],sum_spot_effect['fwhm']+sum_plage_effect['fwhm'],'.',rasterized=True)
    xlabel('BIS SPAN [m/s]')
    ylabel('FWHM [m/s]')
    subplot(338)
    plot(sum_spot_effect['bis_span']+sum_plage_effect['bis_span'],sum_spot_effect['flux']+sum_plage_effect['flux'],'.',rasterized=True)
    xlabel('BIS SPAN [m/s]')
    ylabel("Flux [ppm]")
    subplot(339)
    plot(sum_spot_effect['fwhm']+sum_plage_effect['fwhm'],sum_spot_effect['flux']+sum_plage_effect['flux'],'.',rasterized=True)
    xlabel('FWHM [m/s]')
    ylabel("Flux [ppm]")
    subplots_adjust(top=0.92,left=0.1,bottom=0.1,right=0.92,wspace=0.32,hspace=0.32)
    savefig('figures/SPOT_AND_PLAGE_Correlations_log_Rhk_48_period_'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'.pdf')

    figure(32)
    nb_of_active_regions = len(spot_effect_vect_all['latitude'][:,0])
    lat_active_region,t_active_region = [],[]
    for i in arange(len(spot_effect_vect)):
        lat_active_region.append(spot_effect_vect[i]['latitude'][0])
        t_active_region.append(spot_effect_vect[i]['spot_time'][0])
    plot(t_active_region,lat_active_region,marker='.',ls='',rasterized=True)
    xlabel('Time [d]')
    ylabel('Active region latitude [deg]')
    subplots_adjust(top=0.92,left=0.15,bottom=0.15,right=0.92,wspace=0.32,hspace=0.32)
    savefig('figures/Butterfly_diagram_log_Rhk_48_period_'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'.pdf')


    ##############################################
    ## Writing RV information to file
    ##############################################

    folder = 'results/'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))
    if os.path.exists(folder): 
        pass
    else:
        os.mkdir(folder)
    folder2 = 'results/'+string.replace(str(rotation_period),'.','_')+'_incli_'+str(int(inclinaison))+'/log_Rhk_48'
    if os.path.exists(folder2): 
        pass
    else:
        os.mkdir(folder2)
    filename2 = folder2+'/spot_and_plage_vrad_effect_log_Rhk_48.rdb'
    #selection du format du fichier pour ecrire les donnees
    dd = {'time':(t).tolist()}
    keys = ['flux','vrad','bis_span','fwhm','spot_number','spot_number_visible','spot_area','spot_area_projected']
    dd.update(dict([(ii+'_spot',(sum_spot_effect[ii]).tolist()) for ii in keys]))
    dd.update(dict([(ii+'_plage',(sum_plage_effect[ii]).tolist()) for ii in keys]))
    dd.update({'number_of_active_longitude':number_of_active_longitude_vect.tolist()})
    keys_all = concatenate([['time'],[ii+'_spot' for ii in keys],[ii+'_plage' for ii in keys],['number_of_active_longitude']])
    format = '%.4f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%d\n'

    pyrdb.write_rdb(filename2,dd,keys_all,format) #ecriture les donnees


##############################################
##     Animation to see the spot evolution
##############################################

def generate_animation(spot_effect_vect_all,ratio_plage_spot,t,sampling_t):

    T_star = 5778.    # Effective temperature of the star, 5778 for the Sun, from Planetary Fact Sheets, NASA

    print 'Writing active region movie'

    def loi_Planck(lambda0,Temp):
        c   = 299792458.     # speed of light m/s
        h   = 6.62606896e-34 # Planck constant
        k_b = 1.380e-23      # Boltzmann constant
        return 2*h*pow(c,2)*1./pow(lambda0,5)*1./(exp((h*c)/(lambda0*k_b*Temp))-1);


    shutil.rmtree('animation')
    os.mkdir('animation')

    FFMpegWriter = manimation.writers['ffmpeg']
    metadata = dict(title='Movie Test', artist='Matplotlib',
                    comment='Movie support!')
    writer = FFMpegWriter(fps=5, metadata=metadata)

    animation_frame_rate = 1./(2.*sampling_t) # animation_frame_rate*sampling_t gives you the time in day between 2 images
    animation_time = len(t)/animation_frame_rate

    fig = figure(100,[10.3,10.3],dpi=150)

    # to define the circle
    teta=arange(0,2*pi+0.01,0.01)
    cirle_x=cos(teta)
    cirle_y=sin(teta)
    
    with writer.saving(fig, "animation/active_region.mp4", 150):
        #for i in arange(0,animation_time,animation_frame_rate):
        for i in arange(0,animation_time,animation_frame_rate,'int'):
            
            sys.stdout.flush()
            sys.stdout.write("\r frame =  %i over %i" % (i+1,animation_time))
            
            spot_area_tmp = spot_effect_vect_all['spot_area'][:,i]
            non_zero      = spot_area_tmp.nonzero()

            spot_y_position_tmp = spot_effect_vect_all['y_spot'][:,i][non_zero]
            spot_z_position_tmp = spot_effect_vect_all['z_spot'][:,i][non_zero]
            spot_longitude_tmp  = spot_effect_vect_all['longitude'][:,i][non_zero]%360
            spot_latitude_tmp  = spot_effect_vect_all['latitude'][:,i][non_zero]
            spot_area_tmp    = spot_effect_vect_all['spot_area'][:,i][non_zero]
            
            title('active region simulation')
            #marker size = 100, radius = 70 pixels, area = 5542 pixels, stellar radius 1 = 400 pixels, area = 180956 -> marker size 100 = 17.5% of the stellar radius  -> marker size 1 = 0.175% of the stellar radius
            # If we have a radius of 1, we have to check that the marker fill up the stellar disc. IT DOES
            plot(array([0]),array([0]),marker='o',ms=1/0.00175,color='#E6B800')
            plot(cirle_x,cirle_y,lw=5,color='k')
            xlim(-1.01,1.01)
            ylim(-1.01,1.01)
            
            axis('off')
            
            #marker size = 100, radius = 70 pixels, area = 5542 pixels, stellar radius 1 = 400 pixels, area = 180956 -> marker size 100 = 17.5% of the stellar radius  -> marker size 1 = 0.175% of the stellar radius
            # If we have a radius of 1, we have to check that the marker fill up the stellar disc. IT DOES
            # NOT TOTALLY RIGHT BECAUSE WE PLOT IN 2D and not 3D, THE SIZE OF THE SPOTS/PLAGES ON THE LIMBS ARE NOT CORRECT
            marker_size_spot=sqrt(2*spot_area_tmp*1e-6)/0.00175 # 0.00175 is the radius for a marker of size 1
            marker_size_plage=sqrt(2*(ratio_plage_spot+1)*spot_area_tmp*1e-6)/0.00175# 0.00175 is the radius for a marker of size 1

            for j in arange(len(spot_area_tmp)):

                if spot_area_tmp[j]>0:

                    if spot_longitude_tmp[j]>270 or spot_longitude_tmp[j]<90:
                        mu = abs(cos(spot_longitude_tmp[j]*pi/180.)) #cos(theta), where theta is the angle relative to disc center
                        T_plage = T_star+250.9-407.7*mu+190.9*mu**2.
                        plage_contrast = loi_Planck(5293.4115e-10,T_plage)/loi_Planck(5293.4115e-10,T_star); #the wavelength of the Kitt peak spectrum goes from 3921.2441+6665.5789, the mean being 5293.4115
                        plot(array([spot_y_position_tmp[j]]),array([spot_z_position_tmp[j]]),'o',color='1.0',ms=marker_size_plage[j],alpha = 3*(plage_contrast-1)) #multiply by 3 to see better the effect on the simulation
                        plot(array([spot_y_position_tmp[j]]),array([spot_z_position_tmp[j]]),'o',color='0.1',ms=marker_size_spot[j])

            writer.grab_frame()
            fig.clf()

    close(100)

    print
    print 'Finished, look the result in the animation folder'
    print

